@@ -778,7 +778,7 @@ static char *join_words(int argc, char **argv)
 static int cli_find_command(struct cli_def *cli, struct cli_command *commands, int num_words, char *words[],
                             int start_word, int filters[])
 {
-    struct cli_command *c, *again = NULL;
+    struct cli_command *c, *again_config = NULL, *again_any = NULL;
     int c_words = num_words;
 
     if (filters[0])
@@ -820,7 +820,7 @@ static int cli_find_command(struct cli_def *cli, struct cli_command *commands, i
             continue;
 
         AGAIN:
-        if (c->mode == cli->mode || c->mode == MODE_ANY)
+        if (c->mode == cli->mode || (c->mode == MODE_ANY && again_any != NULL))
         {
             int rc = CLI_OK;
             int f;
@@ -959,17 +959,28 @@ static int cli_find_command(struct cli_def *cli, struct cli_command *commands, i
         {
             // command matched but from another mode,
             // remember it if we fail to find correct command
-            again = c;
+            again_config = c;
+        }
+        else if (c->mode == MODE_ANY)
+        {
+            // command matched but for any mode,
+            // remember it if we fail to find correct command
+            again_any = c;
         }
     }
 
     // drop out of config submode if we have matched command on MODE_CONFIG
-    if (again)
+    if (again_config)
     {
-        c = again;
+        c = again_config;
         cli_set_configmode(cli, MODE_CONFIG, NULL);
         goto AGAIN;
     }
+    if (again_any)
+    {
+        c = again_any;
+        goto AGAIN;
+    }
 
     if (start_word == 0)
         cli_error(cli, "Invalid %s \"%s\"", commands->parent ? "argument" : "command", words[start_word]);
@@ -1017,7 +1028,7 @@ static int cli_get_completions(struct cli_def *cli, const char *command, char **
 {
     struct cli_command *c;
     struct cli_command *n;
-    int num_words, i, k=0;
+    int num_words, save_words, i, k=0;
     char *words[CLI_MAX_LINE_WORDS] = {0};
     int filter = 0;
 
@@ -1025,12 +1036,12 @@ static int cli_get_completions(struct cli_def *cli, const char *command, char **
     while (isspace(*command))
         command++;
 
-    num_words = cli_parse_line(command, words, sizeof(words)/sizeof(words[0]));
+    save_words = num_words = cli_parse_line(command, words, sizeof(words)/sizeof(words[0]));
     if (!command[0] || command[strlen(command)-1] == ' ')
         num_words++;
 
     if (!num_words)
-        return 0;
+        goto out;
 
     for (i = 0; i < num_words; i++)
     {
@@ -1043,7 +1054,7 @@ static int cli_get_completions(struct cli_def *cli, const char *command, char **
         unsigned len = 0;
 
         if (filter < num_words - 1) // filter already completed
-            return 0;
+            goto out;
 
         if (filter == num_words - 1)
             len = strlen(words[num_words-1]);
@@ -1055,7 +1066,7 @@ static int cli_get_completions(struct cli_def *cli, const char *command, char **
         }
 
         completions[k] = NULL;
-        return k;
+        goto out;
     }
 
     for (c = cli->commands, i = 0; c && i < num_words && k < max_completions; c = n)
@@ -1084,6 +1095,10 @@ static int cli_get_completions(struct cli_def *cli, const char *command, char **
         completions[k++] = c->command;
     }
 
+out:
+    for (i = 0; i < save_words; i++)
+        free(words[i]);
+
     return k;
 }
 
